"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const core_1 = require("@salesforce/core");
const chai_1 = require("chai");
const testSetup_1 = require("@salesforce/core/lib/testSetup");
const sinon_1 = require("sinon");
const metadata_registry_1 = require("../../src/metadata-registry");
const metadataApi_1 = require("../../src/client/metadataApi");
const convert_1 = require("../../src/convert");
const assert_1 = require("assert");
const path = require("path");
const i18n_1 = require("../../src/i18n");
const types_1 = require("../../src/client/types");
describe('Metadata Api', () => {
    let mockConnection;
    let sandboxStub = sinon_1.createSandbox();
    const testData = new testSetup_1.MockTestOrgData();
    const registryAccess = new metadata_registry_1.RegistryAccess();
    const rootPath = path.join('file', 'path');
    const props = {
        name: 'myTestClass',
        type: metadata_registry_1.registryData.types.apexclass,
        xml: path.join(rootPath, 'myTestClass.cls-meta.xml'),
        content: path.join(rootPath, 'myTestClass.cls'),
    };
    const component = metadata_registry_1.SourceComponent.createVirtualComponent(props, [
        {
            dirPath: rootPath,
            children: [path.basename(props.xml), path.basename(props.content)],
        },
    ]);
    const deployResult = {
        id: '12345',
        status: types_1.DeployStatus.Succeeded,
        success: true,
        details: {
            componentSuccesses: [
                // @ts-ignore
                {
                    fullName: component.fullName,
                    success: 'true',
                },
            ],
        },
    };
    const sourceDeployResult = {
        id: '12345',
        success: true,
        status: types_1.DeployStatus.Succeeded,
        components: [
            {
                component,
                status: types_1.ComponentStatus.Unchanged,
                diagnostics: [],
            },
        ],
    };
    const testingBuffer = Buffer.from('testingBuffer');
    const deployPath = path.join('file', 'path', 'myTestClass.cls');
    let metadataClient;
    let registryStub = sandboxStub.stub();
    let convertStub = sandboxStub.stub();
    let deployIdStub = sandboxStub.stub();
    beforeEach(() => __awaiter(void 0, void 0, void 0, function* () {
        sandboxStub = sinon_1.createSandbox();
        mockConnection = yield core_1.Connection.create({
            authInfo: yield core_1.AuthInfo.create({
                username: testData.username,
            }),
        });
        metadataClient = new metadataApi_1.MetadataApi(mockConnection, registryAccess);
        registryStub = sandboxStub.stub(registryAccess, 'getComponentsFromPath').returns([component]);
        convertStub = sandboxStub
            .stub(convert_1.MetadataConverter.prototype, 'convert')
            .withArgs(sinon_1.match.any, 'metadata', { type: 'zip' })
            .resolves({
            zipBuffer: testingBuffer,
        });
    }));
    afterEach(() => {
        sandboxStub.restore();
    });
    it('Should check that the default options are correct', () => __awaiter(void 0, void 0, void 0, function* () {
        const defaultOptions = {
            rollbackOnError: true,
            ignoreWarnings: false,
            checkOnly: false,
            singlePackage: true,
        };
        chai_1.expect(metadataApi_1.DEFAULT_API_OPTIONS).to.deep.equal(defaultOptions);
    }));
    it('Should correctly deploy metadata components from paths', () => __awaiter(void 0, void 0, void 0, function* () {
        // @ts-ignore minimum info required
        deployIdStub = sandboxStub.stub(mockConnection.metadata, 'deploy').resolves({
            id: '12345',
        });
        const deployPollStub = sandboxStub
            .stub(mockConnection.metadata, 'checkDeployStatus')
            .withArgs('12345', true)
            // @ts-ignore minimum info required
            .resolves(deployResult);
        const deploys = yield metadataClient.deployWithPaths(deployPath);
        chai_1.expect(registryStub.calledImmediatelyBefore(convertStub)).to.be.true;
        chai_1.expect(convertStub.calledImmediatelyBefore(deployIdStub)).to.be.true;
        chai_1.expect(deployIdStub.calledImmediatelyBefore(deployPollStub)).to.be.true;
        chai_1.expect(deploys).to.deep.equal(sourceDeployResult);
    }));
    it('Should correctly deploy metadata components with custom deploy options', () => __awaiter(void 0, void 0, void 0, function* () {
        const apiOptions = {
            allowMissingFiles: true,
            autoUpdatePackage: true,
            checkOnly: true,
            ignoreWarnings: true,
            performRetrieve: true,
            purgeOnDelete: true,
            rollbackOnError: true,
            runAllTests: true,
            runTests: ['test1', 'test2'],
            singlePackage: true,
        };
        deployIdStub = sandboxStub
            .stub(mockConnection.metadata, 'deploy')
            .withArgs(testingBuffer, apiOptions)
            // @ts-ignore
            .resolves({
            id: '12345',
        });
        // @ts-ignore
        sandboxStub.stub(mockConnection.metadata, 'checkDeployStatus').resolves(deployResult);
        yield metadataClient.deployWithPaths(deployPath, { apiOptions });
        chai_1.expect(deployIdStub.args).to.deep.equal([[testingBuffer, apiOptions]]);
    }));
    it('Should correctly deploy metadata components with default deploy options', () => __awaiter(void 0, void 0, void 0, function* () {
        deployIdStub = sandboxStub
            .stub(mockConnection.metadata, 'deploy')
            .withArgs(testingBuffer, metadataApi_1.DEFAULT_API_OPTIONS)
            // @ts-ignore
            .resolves({
            id: '12345',
        });
        // @ts-ignore
        sandboxStub.stub(mockConnection.metadata, 'checkDeployStatus').resolves(deployResult);
        yield metadataClient.deployWithPaths(deployPath);
        chai_1.expect(deployIdStub.args).to.deep.equal([[testingBuffer, metadataApi_1.DEFAULT_API_OPTIONS]]);
    }));
    it('Should correctly deploy metadata components with default and custom deploy options', () => __awaiter(void 0, void 0, void 0, function* () {
        const apiOptions = {
            rollbackOnError: true,
            ignoreWarnings: false,
            checkOnly: true,
            autoUpdatePackage: true,
            singlePackage: false,
        };
        deployIdStub = sandboxStub
            .stub(mockConnection.metadata, 'deploy')
            .withArgs(testingBuffer, apiOptions)
            // @ts-ignore
            .resolves({
            id: '12345',
        });
        // @ts-ignore
        sandboxStub.stub(mockConnection.metadata, 'checkDeployStatus').resolves(deployResult);
        yield metadataClient.deployWithPaths(deployPath, {
            apiOptions: { checkOnly: true, autoUpdatePackage: true, singlePackage: false },
        });
        chai_1.expect(deployIdStub.args).to.deep.equal([[testingBuffer, apiOptions]]);
    }));
    describe('Metadata Status Poll', () => {
        it('should verify successful status poll', () => __awaiter(void 0, void 0, void 0, function* () {
            // @ts-ignore
            deployIdStub = sandboxStub.stub(mockConnection.metadata, 'deploy').resolves({
                id: '12345',
            });
            sandboxStub
                .stub(mockConnection.metadata, 'checkDeployStatus')
                // @ts-ignore
                .resolves(deployResult);
            const deploys = yield metadataClient.deployWithPaths(deployPath);
            chai_1.expect(deploys).to.deep.equal(sourceDeployResult);
        }));
        it('should throw correct error for unexpected issue', () => __awaiter(void 0, void 0, void 0, function* () {
            // @ts-ignore
            deployIdStub = sandboxStub.stub(mockConnection.metadata, 'deploy').resolves({
                id: '12345',
            });
            sandboxStub.stub(mockConnection.metadata, 'checkDeployStatus').throws('unexpected error');
            try {
                yield metadataClient.deployWithPaths(deployPath);
                assert_1.fail('request should have failed');
            }
            catch (e) {
                chai_1.expect(e.message).contains(i18n_1.nls.localize('md_request_fail', 'unexpected error'));
            }
        }));
        it('should verify timeout status poll', () => __awaiter(void 0, void 0, void 0, function* () {
            // @ts-ignore
            deployIdStub = sandboxStub.stub(mockConnection.metadata, 'deploy').resolves({
                id: '12345',
            });
            const deployOptionWait = {
                wait: 100,
            };
            const deployPollStub = sandboxStub.stub(mockConnection.metadata, 'checkDeployStatus');
            // @ts-ignore
            deployPollStub.resolves({ status: 'Pending', success: false });
            const result = yield metadataClient.deployWithPaths(deployPath, deployOptionWait);
            chai_1.expect(result.status).to.equal('Pending');
            chai_1.expect(result.success).to.be.false;
        }));
    });
    describe('Metadata Deploy Result', () => {
        it('should set deploy operation status correctly', () => __awaiter(void 0, void 0, void 0, function* () {
            // @ts-ignore
            sandboxStub.stub(mockConnection.metadata, 'deploy').resolves({
                id: '12345',
            });
            sandboxStub
                .stub(mockConnection.metadata, 'checkDeployStatus')
                // @ts-ignore minimum info required
                .resolves({
                success: true,
                id: '1234',
                status: 'Succeeded',
            });
            const result = yield metadataClient.deploy(component);
            chai_1.expect(result).to.deep.equal({
                id: '1234',
                success: true,
                status: 'Succeeded',
            });
        }));
        it('should set Changed component status for changed component', () => __awaiter(void 0, void 0, void 0, function* () {
            // @ts-ignore
            sandboxStub.stub(mockConnection.metadata, 'deploy').resolves({
                id: '12345',
            });
            sandboxStub
                .stub(mockConnection.metadata, 'checkDeployStatus')
                // @ts-ignore minimum info required
                .resolves({
                success: true,
                id: '1234',
                status: 'Succeeded',
                details: {
                    // @ts-ignore
                    componentSuccesses: [
                        {
                            changed: 'true',
                            created: 'false',
                            deleted: 'false',
                            fullName: component.fullName,
                            componentType: component.type.name,
                        },
                    ],
                },
            });
            const result = yield metadataClient.deploy(component);
            chai_1.expect(result.components).to.deep.equal([
                {
                    component,
                    status: types_1.ComponentStatus.Changed,
                    diagnostics: [],
                },
            ]);
        }));
        it('should set Created component status for created component', () => __awaiter(void 0, void 0, void 0, function* () {
            // @ts-ignore
            sandboxStub.stub(mockConnection.metadata, 'deploy').resolves({
                id: '12345',
            });
            sandboxStub
                .stub(mockConnection.metadata, 'checkDeployStatus')
                // @ts-ignore minimum info required
                .resolves({
                success: true,
                id: '1234',
                status: 'Succeeded',
                details: {
                    // @ts-ignore
                    componentSuccesses: [
                        {
                            changed: 'false',
                            created: 'true',
                            deleted: 'false',
                            fullName: component.fullName,
                            componentType: component.type.name,
                        },
                    ],
                },
            });
            const result = yield metadataClient.deploy(component);
            chai_1.expect(result.components).to.deep.equal([
                {
                    component,
                    status: types_1.ComponentStatus.Created,
                    diagnostics: [],
                },
            ]);
        }));
        it('should set Deleted component status for deleted component', () => __awaiter(void 0, void 0, void 0, function* () {
            // @ts-ignore
            sandboxStub.stub(mockConnection.metadata, 'deploy').resolves({
                id: '12345',
            });
            sandboxStub
                .stub(mockConnection.metadata, 'checkDeployStatus')
                // @ts-ignore minimum info required
                .resolves({
                success: true,
                id: '1234',
                status: 'Succeeded',
                details: {
                    // @ts-ignore
                    componentSuccesses: {
                        changed: 'false',
                        created: 'false',
                        deleted: 'true',
                        fullName: component.fullName,
                        componentType: component.type.name,
                    },
                },
            });
            const result = yield metadataClient.deploy(component);
            chai_1.expect(result.components).to.deep.equal([
                {
                    component,
                    status: types_1.ComponentStatus.Deleted,
                    diagnostics: [],
                },
            ]);
        }));
        it('should set Failed component status for failed component', () => __awaiter(void 0, void 0, void 0, function* () {
            // @ts-ignore
            sandboxStub.stub(mockConnection.metadata, 'deploy').resolves({
                id: '12345',
            });
            sandboxStub
                .stub(mockConnection.metadata, 'checkDeployStatus')
                // @ts-ignore minimum info required
                .resolves({
                success: true,
                id: '1234',
                status: 'Failed',
                details: {
                    // @ts-ignore
                    componentFailures: {
                        success: 'false',
                        changed: 'false',
                        created: 'false',
                        deleted: 'false',
                        fullName: component.fullName,
                        componentType: component.type.name,
                    },
                },
            });
            const result = yield metadataClient.deploy(component);
            chai_1.expect(result.components).to.deep.equal([
                {
                    component,
                    status: types_1.ComponentStatus.Failed,
                    diagnostics: [],
                },
            ]);
        }));
        it('should aggregate diagnostics for a component', () => __awaiter(void 0, void 0, void 0, function* () {
            // @ts-ignore
            sandboxStub.stub(mockConnection.metadata, 'deploy').resolves({
                id: '12345',
            });
            sandboxStub
                .stub(mockConnection.metadata, 'checkDeployStatus')
                // @ts-ignore minimum info required
                .resolves({
                success: false,
                id: '1234',
                status: 'Failed',
                details: {
                    // @ts-ignore
                    componentFailures: [
                        {
                            success: 'false',
                            changed: 'false',
                            created: 'false',
                            deleted: 'false',
                            fullName: component.fullName,
                            componentType: component.type.name,
                            problem: 'Expected ;',
                            problemType: 'Error',
                            lineNumber: 3,
                            columnNumber: 7,
                        },
                        {
                            success: 'false',
                            changed: 'false',
                            created: 'false',
                            deleted: 'false',
                            fullName: component.fullName,
                            componentType: component.type.name,
                            problem: 'Symbol test does not exist',
                            problemType: 'Error',
                            lineNumber: 8,
                            columnNumber: 23,
                        },
                    ],
                },
            });
            const result = yield metadataClient.deploy(component);
            chai_1.expect(result.components).to.deep.equal([
                {
                    component,
                    status: types_1.ComponentStatus.Failed,
                    diagnostics: [
                        {
                            lineNumber: 3,
                            columnNumber: 7,
                            message: 'Expected ;',
                            type: 'Error',
                        },
                        {
                            lineNumber: 8,
                            columnNumber: 23,
                            message: 'Symbol test does not exist',
                            type: 'Error',
                        },
                    ],
                },
            ]);
        }));
        it('should fix lwc deploy message issue', () => __awaiter(void 0, void 0, void 0, function* () {
            const bundlePath = path.join('path', 'to', 'lwc', 'test');
            const props = {
                name: 'test',
                type: metadata_registry_1.registryData.types.lightningcomponentbundle,
                xml: path.join(bundlePath, 'test.js-meta.xml'),
                content: bundlePath,
            };
            const component = metadata_registry_1.SourceComponent.createVirtualComponent(props, [
                {
                    dirPath: bundlePath,
                    children: [path.basename(props.xml), 'test.js', 'test.html'],
                },
            ]);
            // @ts-ignore
            sandboxStub.stub(mockConnection.metadata, 'deploy').resolves({
                id: '12345',
            });
            sandboxStub
                .stub(mockConnection.metadata, 'checkDeployStatus')
                // @ts-ignore minimum info required
                .resolves({
                success: true,
                id: '1234',
                status: 'Succeeded',
                details: {
                    // @ts-ignore
                    componentSuccesses: [
                        {
                            success: 'false',
                            changed: 'false',
                            created: 'true',
                            deleted: 'false',
                            fullName: 'markup://c:test',
                            componentType: component.type.name,
                        },
                    ],
                },
            });
            const result = yield metadataClient.deploy(component);
            chai_1.expect(result.components).to.deep.equal([
                {
                    component,
                    status: types_1.ComponentStatus.Created,
                    diagnostics: [],
                },
            ]);
        }));
    });
});
//# sourceMappingURL=metadataApi.test.js.map