"use strict";
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
Object.defineProperty(exports, "__esModule", { value: true });
const registry_1 = require("../../mock/registry");
const decomposedMetadataTransformer_1 = require("../../../src/convert/transformers/decomposedMetadataTransformer");
const convertTransaction_1 = require("../../../src/convert/convertTransaction");
const chai_1 = require("chai");
const sinon_1 = require("sinon");
const fs = require("fs");
const constants_1 = require("../../../src/utils/constants");
const stream_1 = require("stream");
const path_1 = require("path");
const errors_1 = require("../../../src/errors");
const i18n_1 = require("../../../src/i18n");
const env = sinon_1.createSandbox();
describe('DecomposedMetadataTransformer', () => {
    const component = registry_1.regina.REGINA_COMPONENT;
    afterEach(() => env.restore());
    it('should register RecompositionFinalizer', () => {
        const transaction = new convertTransaction_1.ConvertTransaction();
        const addFinalizerSpy = env.spy(transaction, 'addFinalizer');
        new decomposedMetadataTransformer_1.DecomposedMetadataTransformer(component, transaction);
        chai_1.expect(addFinalizerSpy.calledWith(convertTransaction_1.RecompositionFinalizer)).to.be.true;
    });
    describe('toMetadataFormat', () => {
        it('should delay for partial recomposition when a child component is given', () => {
            const child = component.getChildren()[0];
            const transaction = new convertTransaction_1.ConvertTransaction();
            const transformer = new decomposedMetadataTransformer_1.DecomposedMetadataTransformer(child, transaction);
            const writerFormat = transformer.toMetadataFormat();
            chai_1.expect(writerFormat).to.deep.equal({ component: child, writeInfos: [] });
            chai_1.expect(transaction.state).to.deep.equal({
                recompose: {
                    [component.fullName]: {
                        component,
                        children: [child],
                    },
                },
            });
        });
        it('should fully recompose metadata when a parent component is given', () => {
            const transformer = new decomposedMetadataTransformer_1.DecomposedMetadataTransformer(component, new convertTransaction_1.ConvertTransaction());
            const children = component.getChildren();
            const readStub = env.stub(fs, 'readFileSync');
            readStub.withArgs(children[0].xml).returns('<Y><test>child1</test></Y>');
            readStub.withArgs(children[1].xml).returns('<X><test>child2</test></X>');
            readStub
                .withArgs(component.xml)
                .returns(`${constants_1.XML_DECL}<ReginaKing xmlns="${constants_1.XML_NS}"><foo>bar</foo></ReginaKing>\n`);
            const result = transformer.toMetadataFormat();
            const expectedStr = `${constants_1.XML_DECL}<ReginaKing xmlns="${constants_1.XML_NS}">\n  <foo>bar</foo>\n  <ys>\n    <test>child1</test>\n  </ys>\n  <xs>\n    <test>child2</test>\n  </xs>\n</ReginaKing>\n`;
            const source = new stream_1.Readable();
            source.push(expectedStr);
            source.push(null);
            chai_1.expect(result).to.deep.equal({
                component,
                writeInfos: [
                    {
                        relativeDestination: path_1.join('reginas', 'a.regina'),
                        source,
                    },
                ],
            });
        });
        describe('toSourceFormat', () => {
            it('should throw a not implemented error', () => {
                const transformer = new decomposedMetadataTransformer_1.DecomposedMetadataTransformer(component, new convertTransaction_1.ConvertTransaction());
                chai_1.assert.throws(() => transformer.toSourceFormat(), errors_1.LibraryError, i18n_1.nls.localize('error_convert_not_implemented', ['source', component.type.name]));
            });
        });
    });
});
//# sourceMappingURL=decomposedMetadataTransformer.js.map