"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2020, salesforce.com, inc.
 * All rights reserved.
 * Licensed under the BSD 3-Clause license.
 * For full license text, see LICENSE.txt file in the repo root or https://opensource.org/licenses/BSD-3-Clause
 */
const staticResourceMetadataTransformer_1 = require("../../../src/convert/transformers/staticResourceMetadataTransformer");
const sinon_1 = require("sinon");
const path_1 = require("path");
const mixedContentSingleFileConstants_1 = require("../../mock/registry/mixedContentSingleFileConstants");
const utils_1 = require("../../../src/utils");
const fs = require("fs");
const chai_1 = require("chai");
const readables_1 = require("../../mock/convert/readables");
const tarajiConstants_1 = require("../../mock/registry/tarajiConstants");
const constants_1 = require("../../../src/utils/constants");
const archiver = require("archiver");
const src_1 = require("../../../src");
const errors_1 = require("../../../src/errors");
const i18n_1 = require("../../../src/i18n");
const env = sinon_1.createSandbox();
describe('StaticResourceMetadataTransformer', () => {
    beforeEach(() => 
    // @ts-ignore mock readable isn't an fs readable specifically
    env.stub(fs, 'createReadStream').callsFake((fsPath) => new readables_1.TestReadable(fsPath)));
    afterEach(() => env.restore());
    describe('toMetadataFormat', () => {
        it('should rename extension to .resource for content file', () => {
            const component = mixedContentSingleFileConstants_1.MC_SINGLE_FILE_COMPONENT;
            const { type, content, xml } = component;
            env.stub(component, 'parseXml').returns({
                StaticResource: {
                    contentType: 'png',
                },
            });
            const transformer = new staticResourceMetadataTransformer_1.StaticResourceMetadataTransformer(component);
            const expectedInfos = [
                {
                    source: fs.createReadStream(content),
                    relativeDestination: path_1.join(type.directoryName, `${utils_1.baseName(content)}.${type.suffix}`),
                },
                {
                    source: fs.createReadStream(xml),
                    relativeDestination: path_1.join(type.directoryName, path_1.basename(xml)),
                },
            ];
            chai_1.expect(transformer.toMetadataFormat()).to.deep.equal({
                component,
                writeInfos: expectedInfos,
            });
        });
        it('should zip directory content for all supported archive mime types', () => {
            const component = src_1.SourceComponent.createVirtualComponent(tarajiConstants_1.TARAJI_COMPONENT, tarajiConstants_1.TARAJI_VIRTUAL_FS);
            const { type, content, xml } = component;
            const archive = archiver.create('zip', { zlib: { level: 3 } });
            const archiveDirStub = env.stub(archive, 'directory');
            const archiveFinalizeStub = env.stub(archive, 'finalize');
            const parseXmlStub = env.stub(component, 'parseXml');
            env.stub(archiver, 'create').returns(archive);
            const transformer = new staticResourceMetadataTransformer_1.StaticResourceMetadataTransformer(component);
            const expectedInfos = [
                {
                    source: archive,
                    relativeDestination: path_1.join(type.directoryName, `${utils_1.baseName(content)}.${type.suffix}`),
                },
                {
                    source: fs.createReadStream(xml),
                    relativeDestination: path_1.join(type.directoryName, path_1.basename(xml)),
                },
            ];
            for (const contentType of constants_1.ARCHIVE_MIME_TYPES) {
                parseXmlStub.returns({ StaticResource: { contentType } });
                chai_1.expect(transformer.toMetadataFormat()).to.deep.equal({
                    component,
                    writeInfos: expectedInfos,
                });
                chai_1.expect(archiveDirStub.calledOnceWith(content, false)).to.be.true;
                chai_1.expect(archiveFinalizeStub.calledImmediatelyAfter(archiveDirStub)).to.be.true;
                archiveDirStub.resetHistory();
            }
        });
        it('should throw an error if content is directory but contentType is not an archive type', () => {
            const component = src_1.SourceComponent.createVirtualComponent(tarajiConstants_1.TARAJI_COMPONENT, tarajiConstants_1.TARAJI_VIRTUAL_FS);
            const contentType = 'nonArchiveType';
            env.stub(component, 'parseXml').returns({ StaticResource: { contentType } });
            const transformer = new staticResourceMetadataTransformer_1.StaticResourceMetadataTransformer(component);
            chai_1.assert.throws(() => transformer.toMetadataFormat(), errors_1.LibraryError, i18n_1.nls.localize('error_static_resource_expected_archive_type', [contentType, component.name]));
        });
    });
    describe('toSourceFormat', () => {
        it('should throw a not implemented error', () => {
            const component = mixedContentSingleFileConstants_1.MC_SINGLE_FILE_COMPONENT;
            const transformer = new staticResourceMetadataTransformer_1.StaticResourceMetadataTransformer(component);
            chai_1.assert.throws(() => transformer.toSourceFormat(), errors_1.LibraryError, i18n_1.nls.localize('error_convert_not_implemented', ['source', component.type.name]));
        });
    });
});
//# sourceMappingURL=staticResourceMetadataTransformer.js.map